'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _inquirer = require('inquirer');

var _inquirer2 = _interopRequireDefault(_inquirer);

var _es6Promisify = require('es6-promisify');

var _es6Promisify2 = _interopRequireDefault(_es6Promisify);

var _semver = require('semver');

var _semver2 = _interopRequireDefault(_semver);

var _child_process = require('child_process');

var _ora = require('ora');

var _ora2 = _interopRequireDefault(_ora);

var _config = require('./config');

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var cwd = process.cwd();
var pkgPath = _path2.default.join(cwd, 'package.json');
var pkg = _fs2.default.readFileSync(pkgPath, 'utf8');
var content = JSON.parse(pkg);
var rNameSpace = /^(@\w+)\//;

var spinner = void 0;

/**
 * 逻辑发生变更
 * 1. 如果没有命名空间，走正常逻辑
 *   - (对同样版本的要进行 tag 的各种移除)
 *   - 变更版本
 *   - 推送代码
 *   - 发布
 * 2. 如果有命名空间
 *   - 检查是否配置了对应命名空间
 *   - 没有配置提示用户配置
 *   - 走正常逻辑
 *   - 发布携带对应的源
 *   - 需要一步同步手段（没配置就不同步了
 */

var RELEASE_TAG = ['ga: 正式版本', 'rc: 发布候选版本, 不会新增 feature', 'beta: 公测版, 会持续增加 feature', 'alpha: 内测版, 拥有成吨的 bug'];

var inc = function inc(version, tag) {
  var method = [{ key: 'major', desc: '大版本升级' }, { key: 'minor', desc: '中版本升级' }, { key: 'patch', desc: '小版本升级' }, { key: 'prerelease', desc: '预升级' }, { key: 'premajor', desc: '大版本预升级' }, { key: 'preminor', desc: '中版本预升级' }, { key: 'prepatch', desc: '小版本预升级' }];

  // 非正式版只能发 pre-tag
  var m = method.filter(function (v) {
    var pre = /^pre/.test(v.key);
    var ga = tag === 'ga';
    return ga ^ pre;
  });

  m.push({ key: 'current', desc: '当前版本' });

  return m.map(function (m) {
    var v = m.key === 'current' ? version : _semver2.default.inc(version, m.key, tag);
    return m.key + ': ' + m.desc + '(' + v + ')';
  });
};

var tagPrompt = [{
  type: 'list',
  name: 'tag',
  message: '请选择要发布的版本类型:',
  default: 0,
  choices: RELEASE_TAG,
  filter: function filter(v) {
    return v.replace(/^(\w+):.+$/, '$1');
  }
}];

var removeGitTag = function () {
  var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(nextRef, remote, github) {
    return _regenerator2.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            spinner = (0, _ora2.default)({ text: '\u79FB\u9664\u672C\u5730 tag: ' + nextRef });
            spinner.start();
            _context.prev = 2;
            _context.next = 5;
            return (0, _es6Promisify2.default)(_child_process.exec)('git tag -d ' + nextRef);

          case 5:
            spinner.succeed();
            _context.next = 12;
            break;

          case 8:
            _context.prev = 8;
            _context.t0 = _context['catch'](2);

            spinner.text = '\u672C\u5730\u4E0D\u5B58\u5728 ' + nextRef + ' tag';
            spinner.stopAndPersist('◎');

          case 12:
            if (!(remote.github !== 'origin')) {
              _context.next = 25;
              break;
            }

            spinner = (0, _ora2.default)({ text: '\u79FB\u9664 origin \u8FDC\u7AEF tag: ' + nextRef });
            spinner.start();
            _context.prev = 15;
            _context.next = 18;
            return (0, _es6Promisify2.default)(_child_process.exec)('git push origin -d tag ' + nextRef);

          case 18:
            spinner.succeed();
            _context.next = 25;
            break;

          case 21:
            _context.prev = 21;
            _context.t1 = _context['catch'](15);

            spinner.text = 'origin \u4E0A\u4E0D\u5B58\u5728 ' + nextRef + ' tag';
            spinner.stopAndPersist('◎');

          case 25:
            if (!(github && remote.github)) {
              _context.next = 38;
              break;
            }

            spinner = (0, _ora2.default)({ text: '\u79FB\u9664 github \u8FDC\u7AEF tag: ' + nextRef });
            spinner.start();
            _context.prev = 28;
            _context.next = 31;
            return (0, _es6Promisify2.default)(_child_process.exec)('git push ' + remote.github + ' -d tag ' + nextRef);

          case 31:
            spinner.succeed();
            _context.next = 38;
            break;

          case 34:
            _context.prev = 34;
            _context.t2 = _context['catch'](28);

            spinner.text = 'github \u4E0A\u4E0D\u5B58\u5728 ' + nextRef + ' tag';
            spinner.stopAndPersist('◎');

          case 38:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, undefined, [[2, 8], [15, 21], [28, 34]]);
  }));

  return function removeGitTag(_x, _x2, _x3) {
    return _ref.apply(this, arguments);
  };
}();

// TODO: 把代码拆一拆
exports.default = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2() {
  var cVersion, _ref3, tag, config, questions, _ref4, version, github, remote, name, nextRef, matched, ns, official, conf, sInfo, tagName, syncChoices, syncPrompt, _ref5, website, urlList;

  return _regenerator2.default.wrap(function _callee2$(_context2) {
    while (1) {
      switch (_context2.prev = _context2.next) {
        case 0:
          cVersion = content.version;
          _context2.next = 3;
          return _inquirer2.default.prompt(tagPrompt);

        case 3:
          _ref3 = _context2.sent;
          tag = _ref3.tag;
          config = (0, _config.getConfig)();

          spinner = (0, _ora2.default)({ text: '检测 npm 源' });

          questions = [{
            type: 'list',
            name: 'version',
            message: '请选择升版方式:',
            default: 0,
            choices: inc(cVersion, tag),
            filter: function filter(v) {
              return v.replace(/^[^(]+\(([\w\-.]+)\)$/, '$1');
            }
          }, {
            type: 'confirm',
            name: 'github',
            message: '是否同步代码到 github (小心安全组)?',
            default: true
          }];
          _context2.prev = 8;
          _context2.next = 11;
          return _inquirer2.default.prompt(questions);

        case 11:
          _ref4 = _context2.sent;
          version = _ref4.version;
          github = _ref4.github;
          _context2.next = 16;
          return (0, _utils.parseRemote)();

        case 16:
          remote = _context2.sent;
          name = content.name;
          nextRef = 'v' + version;
          matched = name.match(rNameSpace);
          ns = matched ? matched[1] : 'npm(official)';
          official = ns === 'npm(official)';
          conf = config.filter(function (v) {
            return v.namespace === ns;
          })[0];

          if (conf) {
            _context2.next = 29;
            break;
          }

          _utils.log.error('\u672A\u68C0\u6D4B\u5230\u540D\u4E3A ' + ns + ' \u7684\u5185\u90E8\u7A7A\u95F4\uFF0C\u8BF7\u8FDB\u884C\u914D\u7F6E');
          _context2.next = 27;
          return (0, _config.add)(ns);

        case 27:
          conf = _context2.sent;

          config = (0, _config.getConfig)();

        case 29:
          if (!(github && !remote.github)) {
            _context2.next = 31;
            break;
          }

          throw new Error('本地无法找到 github 的 remote!');

        case 31:

          spinner.start();

          _context2.prev = 32;
          _context2.next = 35;
          return (0, _es6Promisify2.default)(_child_process.exec)('npm info ' + name + ' --registry=' + conf.registry);

        case 35:
          sInfo = _context2.sent;

          /* eslint-disable no-eval */
          eval('global.npmInfo=' + sInfo);
          /* eslint-enable no-eval */
          _context2.next = 42;
          break;

        case 39:
          _context2.prev = 39;
          _context2.t0 = _context2['catch'](32);

          // npm 源上根本没有这个项目
          global.npmInfo = null;

        case 42:
          if (!(global.npmInfo && global.npmInfo.versions.indexOf(version) > -1)) {
            _context2.next = 56;
            break;
          }

          if (!official) {
            _context2.next = 47;
            break;
          }

          throw new Error('npm \u6E90\u4E0A\u5DF2\u5B58\u5728 ' + version + ' \u7248\u672C, \u8BF7\u4E0D\u8981\u91CD\u590D\u53D1\u5E03!');

        case 47:
          spinner.text = '\u79FB\u9664\u79C1\u6709\u6E90\u4E0A\u5DF2\u53D1\u5E03\u7684\u7248\u672C ' + name + '@' + version;
          _context2.next = 50;
          return (0, _es6Promisify2.default)(_child_process.exec)('npm unpublish ' + name + '@' + version + ' --registry=' + conf.registry);

        case 50:
          spinner.text = name + '@' + version + ' \u5DF2 unpublish';
          spinner.succeed();

          _context2.next = 54;
          return removeGitTag(nextRef, remote, github);

        case 54:
          _context2.next = 69;
          break;

        case 56:
          // 检查 npm 源成功
          spinner.succeed();
          spinner = (0, _ora2.default)({ text: '\u66F4\u65B0 package.json \u5230: ' + version });
          spinner.start();
          _context2.next = 61;
          return (0, _utils.updateVersion)(version);

        case 61:
          _context2.prev = 61;
          _context2.next = 64;
          return (0, _es6Promisify2.default)(_child_process.exec)('git commit -am "chore: Version to ' + version + '"');

        case 64:
          _context2.next = 68;
          break;

        case 66:
          _context2.prev = 66;
          _context2.t1 = _context2['catch'](61);

        case 68:
          spinner.succeed();

        case 69:
          _context2.prev = 69;
          _context2.next = 72;
          return (0, _es6Promisify2.default)(_child_process.exec)('git tag ' + nextRef);

        case 72:
          _context2.next = 84;
          break;

        case 74:
          _context2.prev = 74;
          _context2.t2 = _context2['catch'](69);

          if (!/exists/.test(_context2.t2.message)) {
            _context2.next = 83;
            break;
          }

          _context2.next = 79;
          return removeGitTag(nextRef, remote, github);

        case 79:
          _context2.next = 81;
          return (0, _es6Promisify2.default)(_child_process.exec)('git tag ' + nextRef);

        case 81:
          _context2.next = 84;
          break;

        case 83:
          throw _context2.t2;

        case 84:
          if (!(remote.github !== 'origin')) {
            _context2.next = 92;
            break;
          }

          spinner = (0, _ora2.default)({ text: '\u63A8\u9001\u672C\u5730\u4EE3\u7801\u5230 origin' });
          spinner.start();

          _context2.next = 89;
          return (0, _es6Promisify2.default)(_child_process.exec)('git push origin');

        case 89:
          _context2.next = 91;
          return (0, _es6Promisify2.default)(_child_process.exec)('git push origin ' + nextRef);

        case 91:

          spinner.succeed();

        case 92:
          if (!(github && remote.github)) {
            _context2.next = 100;
            break;
          }

          spinner = (0, _ora2.default)({ text: '\u63A8\u9001\u672C\u5730\u4EE3\u7801\u5230 github' });
          spinner.start();

          _context2.next = 97;
          return (0, _es6Promisify2.default)(_child_process.exec)('git push ' + remote.github);

        case 97:
          _context2.next = 99;
          return (0, _es6Promisify2.default)(_child_process.exec)('git push ' + remote.github + ' ' + nextRef);

        case 99:

          spinner.succeed();

        case 100:
          tagName = tag !== 'ga' ? ' --tag ' + tag : '';


          spinner = (0, _ora2.default)({ text: '\u53D1\u5E03\u65B0\u7248\u672C ' + version + ' \u5230 npm \u6E90' });
          spinner.start();

          _context2.next = 105;
          return (0, _es6Promisify2.default)(_child_process.exec)('npm publish --registry=' + conf.registry + tagName);

        case 105:

          spinner.succeed();

          // 只有发到官网才需要同步内网源

          if (!official) {
            _context2.next = 131;
            break;
          }

          syncChoices = config.filter(function (v) {
            return v.namespace !== 'npm(official)';
          });
          syncPrompt = [{
            type: 'checkbox',
            name: 'website',
            message: '请选择要同步的私有源:',
            default: 0,
            choices: syncChoices.map(function (v) {
              return v.namespace + '(' + v.website + ')';
            })
          }];
          _context2.next = 111;
          return _inquirer2.default.prompt(syncPrompt);

        case 111:
          _ref5 = _context2.sent;
          website = _ref5.website;
          urlList = website.map(function (v) {
            return v.replace(/^[^(]+\(([^)]+)\)$/, '$1');
          });


          spinner = (0, _ora2.default)({ text: '\u540C\u6B65\u5230\u5185\u7F51\u6E90' });
          spinner.start();

          if (urlList.length) {
            _context2.next = 121;
            break;
          }

          spinner.text = '未选择内网源';
          spinner.stopAndPersist('◎');
          _context2.next = 131;
          break;

        case 121:
          _context2.prev = 121;
          _context2.next = 124;
          return (0, _utils.sync)(name, urlList);

        case 124:
          spinner.succeed();
          _context2.next = 131;
          break;

        case 127:
          _context2.prev = 127;
          _context2.t3 = _context2['catch'](121);

          // 同步失败根本无所谓啦
          spinner.text = '同步内网源失败，请检查当前是否是内网环境';
          spinner.fail();

        case 131:

          _utils.log.done('\u7248\u672C ' + version + ' \u53D1\u5E03\u6210\u529F!');
          _utils.log.done((0, _utils.goodbye)());
          _context2.next = 139;
          break;

        case 135:
          _context2.prev = 135;
          _context2.t4 = _context2['catch'](8);

          spinner.text = _context2.t4.message;
          spinner.fail();

        case 139:
        case 'end':
          return _context2.stop();
      }
    }
  }, _callee2, undefined, [[8, 135], [32, 39], [61, 66], [69, 74], [121, 127]]);
}));